# Generated by Django 5.2.8 on 2025-11-24 15:29

import annoying.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("dcim", "0215_rackreservation_status"),
    ]

    operations = [
        migrations.CreateModel(
            name="SmartOSReport",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("report", models.JSONField(default=dict)),
                ("last_heartbeat", models.DateTimeField(null=True)),
                ("last_processed", models.DateTimeField(null=True)),
                (
                    "device",
                    annoying.fields.AutoOneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="smartos_report",
                        to="dcim.device",
                    ),
                ),
            ],
        ),
    ]
